clear 
% Alex Martin, Luca Guerrieri, Michele Modugno: 1/29/2019

% To do: 
% 1) for production purposes, turn the data import section that is now
%    going through an excel spreadsheet into its own script that pulls from
%    Fame directly.
% 2) verify that the unemployment rate is a quarterly average of the
%    monthly series
%
% This program estimates two quantile regression models as described in Section 3
% of the memo entitled "Macroeconomic Implications of the Rise in
% Business-Sector Leverage"
% The first model regresses the change in the unemployment rate on
% the growth rate of the aggregate-nonfinancial-business credit-to-GDP
% ratio (and other macro controls).
% The second model includes separate measures for business and household
% credit.
% The program outputs four CSV files -- see the last section of the
% program. These files are used to create charts using R.



LHS_GDP_switch = 0;   % set to 1 if you want to run a regression for GDP growth instead of the unemployment rate 

boot_n_reps = 1000;  % set to 1 if bootstrap confidence intervals are not needed
                     % set to 1000 to replicate Figure 5 of the memo

unemployment_horizon = 5; % This is the window for the change in the unemployment rate

bandwidth_aggregate_credit = .24;          % bandwidth for the kernel smoothing function, used to infer the density from the quantiles forecasted
bandwidth_business_credit = .23;
bandwidth_business_credit_comparison= .35;  

block_size = unemployment_horizon;  % for block bootstrap -- size of block

credit_to_GDP_growth_horizon = 16;  % window for the growth rate of the credit-to-GDP ratio

if credit_to_GDP_growth_horizon<=unemployment_horizon
    error('The credit_to_GDP_growth_horizon needs to be greater than the unemployment_horizon')
end

comparison_date = '20070930';  % To Do: use year and quarter instead of something with a day.
                               %%this is the end-of-quarter date for the quarter you'd like to compare
                               %the most current forecast to; used to
                               %replicate Figure 8 of the memo
                               
%% clear all output csv files -- this step is taken to avoid accidentally using an old file
delete *csv

%% read data from xlsx spreadsheet
[A,B]=xlsread('c2gdp_data');

%% data manipulation

% dates
tt=A(:,strmatch( 'date', B,'exact'))+datenum('30-Dec-1899');

% unemployment
unemp=A(:,strmatch( 'unemployment', B,'exact'));

gdp=A(:,strmatch('GDP', B,'exact'));

% credit-to-gdp ratio
c2gdp=A(:,strmatch( 'c2gdp', B,'exact'));

% nonfinancial business credit-to-GDP ratio
nfbus=A(:,strmatch( 'nfbus_ratio_lv', B,'exact'));

% houshold credit-to-GDP ratio
hh=A(:,strmatch( 'hh_ratio_lv', B,'exact'));

% PCE deflator
pce_deflator=A(:,strmatch( 'pce_deflator', B,'exact'));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% data manipulations (and line up data length)

% create four-year growth rates for the credit-to-GDP ratio measures
% NB: we lose the first n observations in the process, where n matches credit_to_GDP_growth_horizon

credit_growth=@(credit_measure)(log(credit_measure(credit_to_GDP_growth_horizon+1:end,:))...
                                -log(credit_measure(1:end-credit_to_GDP_growth_horizon,:)))*100;
dc2gdp=credit_growth(c2gdp);
dnfbus=credit_growth(nfbus);
dhh=credit_growth(hh);

% m-quarter change in the unemployment rate, where m is the unemployment
% horizon (from which we lose m observations), then drop another m-n
% observations to match the credit_growth series
dunemp=unemp(unemployment_horizon+1:end,:)-unemp(1:end-unemployment_horizon,:);

dgdp=100*(log(gdp(unemployment_horizon+1:end,:))-log(gdp(1:end-unemployment_horizon,:)));
dgdp1q=100*(log(gdp(2:end,:))-log(gdp(1:end-1,:)));
dgdp1q_lagged = dgdp1q(1:end-unemployment_horizon+1);
truncate_observations = credit_to_GDP_growth_horizon-unemployment_horizon;
dunemp=dunemp(truncate_observations+1:end,:);
dgdp = dgdp(truncate_observations+1:end,:);

% define inflation (m-quarter log change in the PCE deflator)
dpce_deflator = (log(pce_deflator(unemployment_horizon+1:end,:))-log(pce_deflator(1:end-unemployment_horizon,:)))*100;
dpce_deflator = dpce_deflator(truncate_observations+1:end,:);
dgdp1q_lagged=dgdp1q_lagged(truncate_observations+1:end,:);

% Truncate the unemployment series and the time stamps to match the new
% series define above
unemp = unemp(credit_to_GDP_growth_horizon+1:end,:);

tt=tt(credit_to_GDP_growth_horizon+1:end,:);


if LHS_GDP_switch
unemp = dgdp1q_lagged;
dunemp = dgdp;
end
%% Lag all regressors and truncate the change in the unemployment (LHS variable)
lag_function=@(series)series(1:end-unemployment_horizon,:);
truncate_function=@(series)series(unemployment_horizon+1:end,:);
 
% dunemp_lag = lag_function(dunemp);
% unemp_lag = lag_function(unemp);
% dpce_deflator = lag_function(dpce_deflator(1:end-4,:);
% dc2gdp=dc2gdp(1:end-4,:);
% dnfbus=dnfbus(1:end-4,:);
% dhh=dhh(1:end-4,:);
% dunemp_h=dunemp(5:end,:);
% tt=tt(5:end,:);

T=size(dunemp,1);
Q=[0.1:.1:.9];

nq=size(Q,2);

X=[ones(T,1) unemp dpce_deflator dc2gdp];
bq = zeros(size(X,2),nq);
bq_upper = bq;
bq_lower = bq;
for q=1:nq
    
    % alternative function for quantile regression -- does not do bootstrap
    [bq(:,q),stats]=rq_bbb(lag_function(X), truncate_function(dunemp), Q(q),boot_n_reps, block_size);
    
     if boot_n_reps >1
     bq_sorted = sort(stats.pboot');
     bq_upper(:,q) = bq_sorted(ceil(.95*size(bq_sorted,1)),:);
     bq_lower(:,q) = bq_sorted(ceil(.05*size(bq_sorted,1)),:);
     end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% plot coefficients

figure(1)
subplot(2,2,1)
plot(Q,bq(2,:))
ylabel('Coef. of lagged unemployment level')
xlabel('Quantiles')

if boot_n_reps>1
hold on
plot(Q,bq_upper(2,:),'r:')
legend('Point Estimate','95% robust confidence bands')
plot(Q,bq_lower(2,:),'r:')
end

subplot(2,2,2)
plot(Q,bq(3,:))
ylabel('Coef. of lagged inflation')
xlabel('Quantiles')

if boot_n_reps>1
hold on
plot(Q,bq_upper(3,:),'r:')
plot(Q,bq_lower(3,:),'r:')
end

subplot(2,2,3),plot(Q,bq(4,:))
ylabel('Coef. of credit-to-GDP ratio growth rate')
xlabel('Quantiles')

if boot_n_reps >1
hold on
plot(Q,bq_upper(4,:),'r:')
plot(Q,bq_lower(4,:),'r:')

% for output to R, for fancy charts
quantile_coefficients_aggregate =[Q' bq(2,:)' bq_upper(2,:)' bq_lower(2,:)'...
    bq(3,:)' bq_upper(3,:)' bq_lower(3,:)'...
    bq(4,:)' bq_upper(4,:)' bq_lower(4,:)'];
end


figure(3)
predicted_unemployment_change = X*bq;

% plot the forecast based on the last available (original, unlagged)
% observations
[densityu,ugrid] = ksdensity(predicted_unemployment_change(end,:) ,'Bandwidth',bandwidth_aggregate_credit);
plot(ugrid+unemp(end),densityu,'r-'); hold on
forecast_density_aggregate_model = [(ugrid+unemp(end))' densityu'];


figure(4)
plot(ugrid,densityu,'r-'); hold on
median_pos = find(cumsum(densityu/sum(densityu))>0.5,1,'first');
vline(ugrid(median_pos),'r--')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% next regression, include household and business credit separately



X=[ones(T,1) unemp dpce_deflator dnfbus dhh];

bq = zeros(size(X,2),nq);
bq_upper = bq;
bq_lower = bq;

for q=1:nq
    [bq(:,q),stats]=rq_bbb(lag_function(X), truncate_function(dunemp), Q(q), boot_n_reps, block_size);
    
    % compute bootstrap standard errors (using block bootstrap)
    %[bq3(:,q),stats]=quantreg_bbb(lag_function(X), truncate_function(dunemp),Q(q),[],boot_n_reps,block_size);
    if boot_n_reps >1
     bq_sorted = sort(stats.pboot');
     bq_upper(:,q) = bq_sorted(ceil(.95*size(bq_sorted,1)),:);
     bq_lower(:,q) = bq_sorted(ceil(.05*size(bq_sorted,1)),:);
    end
end

% collect data for exporting to R for fancy charts
quantile_coefficients_disaggregate =[Q' bq(2,:)' bq_upper(2,:)' bq_lower(2,:)'...
               bq(3,:)' bq_upper(3,:)' bq_lower(3,:)'...
               bq(4,:)' bq_upper(4,:)' bq_lower(4,:)'...
               bq(5,:)' bq_upper(5,:)' bq_lower(5,:)'];


%% plot coefficients

figure(2)
subplot(2,2,1)
plot(Q,bq(2,:))
ylabel('Coef. of lagged unemployment level')
xlabel('Quantiles')

if boot_n_reps>1
hold on
plot(Q,bq_upper(2,:),'r:')
legend('Point Estimate','95% robust confidence bands')
plot(Q,bq_lower(2,:),'r:')
end

subplot(2,2,2)
plot(Q,bq(3,:))
ylabel('Coef. of lagged inflation')
xlabel('Quantiles')

if boot_n_reps>1
hold on
plot(Q,bq_upper(3,:),'r:')
plot(Q,bq_lower(3,:),'r:')
end

subplot(2,2,3),plot(Q,bq(4,:))
ylabel('Coef. of business credit-to-GDP ratio growth rate')
xlabel('Quantiles')

if boot_n_reps >1
hold on
plot(Q,bq_upper(4,:),'r:')
plot(Q,bq_lower(4,:),'r:')

end

subplot(2,2,4),plot(Q,bq(5,:))
ylabel('Coef. of household credit-to-GDP ratio growth rate')
xlabel('Quantiles')

if boot_n_reps >1
hold on
plot(Q,bq_upper(5,:),'r:')
plot(Q,bq_lower(5,:),'r:')

end


%% Plot forecast densities
figure(3)
predicted_unemployment_change = X*bq;
[densityu,ugrid] = ksdensity(predicted_unemployment_change(end,:) ,'Bandwidth',bandwidth_business_credit);

plot(ugrid+unemp(end),densityu,'b-')
legend('Based on Aggregate Credit','Based on Business and Household Credit')

% data for exporting to R
forecast_density_disaggregate_model=[(ugrid+unemp(end))' densityu'];


figure(4)
plot(ugrid,densityu,'b-')
legend('Based on Aggregate Credit','Based on Business and Household Credit')
median_pos = find(cumsum(densityu/sum(densityu))>0.5,1,'first');
ninetieth_pos = find(cumsum(densityu/sum(densityu))>0.9,1,'first');

ugrid(ninetieth_pos)

vline(ugrid(median_pos),'b--')

densityfunction = densityu/sum(densityu);
mean_forecast = sum(ugrid.*densityfunction)

%% Generate figure 8 of the memo using comparison_date
figure(5)
plot(ugrid,densityu,'b-') 
hold on
vline(ugrid(median_pos),'b--')
%%find index of comparison event
comparison_datenum = datenum(comparison_date, 'yyyymmdd');
comparison_index=find(tt==comparison_datenum);

[densityu,ugrid] = ksdensity(predicted_unemployment_change(comparison_index,:) ,'Bandwidth',bandwidth_business_credit_comparison);

%%plot comparison data
plot(ugrid,densityu,'g-') 
median_pos = find(cumsum(densityu/sum(densityu))>0.5,1,'first');
vline(ugrid(median_pos),'g--')
%data for exporting to R
forecast_density_disaggregate_model_comparison=[(ugrid)' densityu'];
%%
figure
tt_poorman = year(datetime(datestr(tt)))+(month(datetime(datestr(tt)))-3)/12;
plot(tt_poorman+1,predicted_unemployment_change(:,1)+unemp,'r:'); hold on
plot(tt_poorman+1,predicted_unemployment_change(:,3)+unemp,'r--')
plot(tt_poorman+1,predicted_unemployment_change(:,6)+unemp,'r--')
plot(tt_poorman+1,predicted_unemployment_change(:,9)+unemp,'r:')
plot(tt_poorman,unemp,'g-')
ylims = ylim;

nberdates(ylims(1),ylims(2))

figure

plot(tt_poorman,dc2gdp,'k'); hold on
plot(tt_poorman,dnfbus,'r--')
plot(tt_poorman,dhh,'b-.')
legend('Aggregate','Business','Household')
ylims = ylim;

nberdates(ylims(1),ylims(2))






